import Live
from consts import *
import sys
from PhotonX25BankController import PhotonX25BankController
from PhotonX25TrackController import PhotonX25TrackController
from PhotonX25TransportController import PhotonX25TransportController
from PhotonX25FXController import PhotonX25FXController

"""
based on http://bl0rg.net/~manuel/fcb1010-mappings/
"""

class PhotonX25:
	__module__ = __name__
	__doc__ = 'Automap script for PhotonX25 controllers'
	bank_idx=0

	def __init__(self, c_instance):
		PhotonX25.realinit(self, c_instance)

	def realinit(self, c_instance):
		self.c_instance = c_instance
		self.transport_controller  = PhotonX25TransportController(self)
		self.bank_controller  = PhotonX25BankController(self)
		self.track_controller = PhotonX25TrackController(self)
		self.fx_controller  = PhotonX25FXController(self)
		self.components = [self.transport_controller,  self.bank_controller, self.track_controller, self.fx_controller]
		
		self.request_rebuild_midi_map()

	def disconnect(self):
	#	self.song().remove_tracks_listener(self.on_tracks_changed)
		for c in self.components:
			c.disconnect()

	def application(self):
		return Live.Application.get_application()

	def song(self):
		return self.c_instance.song()

	def suggest_input_port(self):
		return str('')

	def suggest_output_port(self):
		return str('')

	def can_lock_to_devices(self):
		return False

	def lock_to_device(self, device):
		pass

	def unlock_to_device(self, device):
		pass
		
	def set_appointed_device(self, device):
		pass

	def toggle_lock(self):
		self.c_instance.toggle_lock()

	def suggest_map_mode(self, cc_no):
		return Live.MidiMap.MapMode.absolute

	def restore_bank(self, bank):
		pass

	def show_message(self, message):
		self.c_instance.show_message(message)

	def instance_identifier(self):
		return self.c_instance.instance_identifier()

	def connect_script_instances(self, instanciated_scripts):
		pass

	def request_rebuild_midi_map(self):
		self.c_instance.request_rebuild_midi_map()

	def send_midi(self, midi_event_bytes):
		self.c_instance.send_midi(midi_event_bytes)

	def refresh_state(self):
		for c in self.components:
			c.refresh_state()

	def build_midi_map(self, midi_map_handle):
		script_handle = self.c_instance.handle()
		for c in self.components:
			c.build_midi_map(script_handle, midi_map_handle)

	def update_display(self):
		for c in self.components:
			c.update_display()
		
	def receive_midi(self, midi_bytes):
		channel = (midi_bytes[0] & CHAN_MASK)
		status = (midi_bytes[0] & STATUS_MASK)
		if (status == CC_STATUS):
			cc_no = midi_bytes[1]
			cc_value = midi_bytes[2]
			for c in self.components:
				c.receive_midi_cc(channel, cc_no, cc_value)
		elif (status == NOTEON_STATUS) or (status == NOTEOFF_STATUS):
			note_no = midi_bytes[1]
			note_vel = midi_bytes[2]
			for c in self.components:
				c.receive_midi_note(channel, status, note_no, note_vel)
		else:
			assert False, ('Unknown MIDI message %s' % str(midi_bytes))

	#def on_tracks_changed(self):
	#	self.log("tracks changed")

