import Live
from PhotonX25Component import PhotonX25Component

from consts import *

class PhotonX25BankController(PhotonX25Component):
	__module__ = __name__
	__doc__ = "Track Bank controller for PhotonX25 remote control surface"
	__filter_funcs__ = ["update_display", "log"]
	bank_width=BANK_WIDTH #number of track per bank (five in our case)
	bank_idx=0	#current bank

	def __init__(self, parent):
		PhotonX25Component.realinit(self, parent)

	def build_midi_map(self, script_handle, midi_map_handle):
		def forward_cc(chan, cc):
			Live.MidiMap.forward_midi_cc(script_handle, midi_map_handle, chan, cc)
		def forward_note(chan, note_no):
			Live.MidiMap.forward_midi_note(script_handle, midi_map_handle, chan, note_no)

		for channel in BTN_CHANNELS:
			for cc_no in BANK_BTNS:
				forward_cc(channel, cc_no)


	def receive_midi_note(self, channel, status, note_no, note_vel):
		pass

	def receive_midi_cc(self, channel, cc_no, cc_value):
		if(channel in BTN_CHANNELS and cc_no in BANK_BTNS):
			if(cc_no == BANK_NEXT and cc_value == BTN_ON):
				self.select_right_bank()
			elif(cc_no == BANK_PREV and cc_value == BTN_ON):
				self.select_left_bank()


	def get_current_bank_idx(self):
		return self.bank_idx


	def get_track_in_bank(self,offset):
		if(self.bank_width*self.bank_idx+offset < len(self.song().tracks) ):
			return(self.parent.song().tracks[self.bank_width*self.bank_idx+offset])
		else:
			return(None)
			

	def select_left_bank(self):
		if(self.bank_idx >0 ):
			self.bank_idx = self.bank_idx - 1

		if( self.bank_idx*self.bank_width + self.bank_width  >len(self.parent.song().tracks) -1):
			self.parent.song().view.selected_track=self.parent.song().tracks[len(self.parent.song().tracks)-1]
		else:
			self.parent.song().view.selected_track=self.get_track_in_bank(self.bank_width)

		self.parent.song().view.selected_track=self.get_track_in_bank(0)


	def select_right_bank(self):
		if(self.bank_idx*self.bank_width + self.bank_width < len(self.parent.song().tracks)-1 ):
			self.bank_idx = self.bank_idx + 1 

		if( self.bank_idx*self.bank_width + self.bank_width >len(self.parent.song().tracks)-1):
			self.parent.song().view.selected_track=self.parent.song().tracks[len(self.parent.song().tracks)-1]
		else:
			self.parent.song().view.selected_track=self.get_track_in_bank(self.bank_width)

		self.parent.song().view.selected_track=self.get_track_in_bank(0)


	def disconnect(self):
		pass
	