import Live
from PhotonX25Component import PhotonX25Component

from consts import *

class PhotonX25FXController(PhotonX25Component):
	__module__ = __name__
	__doc__ = "Device controller for PhotonX25 remote control surface"
	__filter_funcs__ = ["update_display", "log"]

	def __init__(self, parent):
		PhotonX25Component.realinit(self, parent)

	def build_midi_map(self, script_handle, midi_map_handle):
		def forward_cc(chan, cc):
			Live.MidiMap.forward_midi_cc(script_handle, midi_map_handle, chan, cc)
		def forward_note(chan, note_no):
			Live.MidiMap.forward_midi_note(script_handle, midi_map_handle, chan, note_no)

		for channel in BTN_CHANNELS:
				for cc_no in FX_BTNS:
					forward_cc(channel, cc_no)

		for channel in LAYER_C_CHANNELS:
				for cc_no in KNOBS:
					forward_cc(channel, cc_no)
		

	def receive_midi_note(self, channel, status, note_no, note_vel):
		pass

	def receive_midi_cc(self, channel, cc_no, cc_value):
		def index_of(list, elt):
			for i in range(0,len(list)):
				if (list[i] == elt):
					return i

		if(channel in BTN_CHANNELS and cc_no in FX_BTNS and cc_value == BTN_ON):
			self.select_next_fx(index_of(FX_BTNS,cc_no))
		elif(channel in LAYER_C_CHANNELS and cc_no in KNOBS):
			self.set_selected_track_selected_device_value(index_of(KNOBS,cc_no),cc_value)

	
	# SELECT DEVICE ON TRACK
	def select_next_fx(self,offset):
		def index_of(list, elt):
			for i in range(0,len(list)):
				if (list[i] == elt):
					return i

		if(self.parent.bank_controller.get_track_in_bank(offset) == self.selected_track()):
			if(len(self.selected_track().devices) > 0 ):
				if(self.selected_track().view.selected_device == None):
					#select first device
					self.selected_track().view.selected_device=self.selected_track().devices[0]
				else:
					#increment device
					if(index_of(self.selected_track().devices,self.selected_track().view.selected_device) < len(self.selected_track().devices) ):
						self.selected_track().view.selected_device = self.selected_track().devices[index_of(self.selected_track().devices,self.selected_track().view.selected_device)+1]
					else:
						self.selected_track().view.selected_device = self.selected_track().devices[0]
		else:
			#select track
			self.parent.song().view.selected_track = self.parent.bank_controller.get_track_in_bank(offset)
			if(self.selected_track().view.selected_device == None and len(self.selected_track().devices) > 0 ):
				#select first device
				self.selected_track().view.selected_device = self.selected_track().devices[0]


	# SET DEVICE PARAMETER
	def set_selected_track_selected_device_value(self,idx,value):
		if(self.selected_track()!=None):
			if(self.selected_track().view.selected_device!=None):
				self.set_device_value(self.selected_track().view.selected_device,idx,value)

	def set_device_value(self,device,idx,value):
		self.set_device_parameter_value(device.parameters[idx],value)

	def set_device_parameter_value(self,device_parameter,value):
		device_parameter.value=(device_parameter.max - device_parameter.min) * value / 127 + device_parameter.min

	def disconnect(self):
		pass
	