import Live
from PhotonX25Component import PhotonX25Component

from consts import *

class PhotonX25TrackController(PhotonX25Component):
	__module__ = __name__
	__doc__ = "Track controller for PhotonX25 remote control surface"
	__filter_funcs__ = ["update_display", "log"]

	def __init__(self, parent):
		PhotonX25Component.realinit(self, parent)


	def build_midi_map(self, script_handle, midi_map_handle):
		def forward_cc(chan, cc):
			Live.MidiMap.forward_midi_cc(script_handle, midi_map_handle, chan, cc)
		def forward_note(chan, note_no):
			Live.MidiMap.forward_midi_note(script_handle, midi_map_handle, chan, note_no)

		#level and pan
		for channel in LAYER_A_CHANNELS:
				for cc_no in KNOBS:
					forward_cc(channel, cc_no)
					
		for channel in BTN_CHANNELS:
			for cc_no in TRACK_BUTTONS:
				forward_cc(channel, cc_no)
				
		#send a & b
		for channel in LAYER_B_CHANNELS:
				for cc_no in KNOBS:
					forward_cc(channel, cc_no)

	def receive_midi_note(self, channel, status, note_no, note_vel):
		pass

	def receive_midi_cc(self, channel, cc_no, cc_value):

		if (channel in LAYER_A_CHANNELS and cc_no in UPPER_SEMI_LAYER):
				self.set_volume_bank_track(index_of(UPPER_SEMI_LAYER,cc_no),cc_value)
		elif (channel in LAYER_A_CHANNELS and cc_no in LOWER_SEMI_LAYER):
				self.set_pan_bank_track(index_of(LOWER_SEMI_LAYER,cc_no),cc_value)
		elif (channel in LAYER_B_CHANNELS and cc_no in UPPER_SEMI_LAYER):
				self.set_send_level_bank_track(0,index_of(UPPER_SEMI_LAYER,cc_no),cc_value)
		elif (channel in LAYER_B_CHANNELS and cc_no in LOWER_SEMI_LAYER):
				self.set_send_level_bank_track(1,index_of(LOWER_SEMI_LAYER,cc_no),cc_value)
		elif(cc_no in TRACK_BUTTONS):
			if(cc_no == MUTE_TRACK and cc_value == BTN_ON):
				self.toggle_mute_selected_track()
			elif(cc_no == ARM_TRACK and cc_value == BTN_ON):
				self.toggle_arm_selected_track()
			elif(cc_no == SOLO_TRACK and cc_value == BTN_ON):
				self.toggle_solo_selected_track()
			elif(cc_no == NEXT_TRACK and cc_value == BTN_ON):
				self.select_next_track()
			elif(cc_no == PREV_TRACK and cc_value == BTN_ON):
				self.select_prev_track()
				
	
	# VOLUME
	def set_volume_bank_track(self,offset,cc_value):
		self.set_volume_track(self.parent.bank_controller.get_track_in_bank(offset),cc_value)

	def set_volume_selected_track(self,cc_value):
		self.set_volume_track(self.selected_track(),cc_value)

	def set_volume_track(self,track,cc_value):
		track.mixer_device.volume.value=(track.mixer_device.volume.max - track.mixer_device.volume.min)	* cc_value / 127 + track.mixer_device.volume.min


	# PAN 
	def set_pan_bank_track(self,offset,cc_value):
		self.set_pan_track(self.parent.bank_controller.get_track_in_bank(offset),cc_value)

	def set_pan_selected_track(self,cc_value):
		self.set_pan_track(self.selected_track(),cc_value)

	def set_pan_track(self,track,cc_value):
		track.mixer_device.panning.value=(track.mixer_device.panning.max - track.mixer_device.panning.min)	* cc_value / 127 + track.mixer_device.panning.min

	
	# SEND LEVEL
	def set_send_level_bank_track(self,send_no,offset,cc_value):
		self.set_send_level_track(send_no,self.parent.bank_controller.get_track_in_bank(offset),cc_value)

	def set_send_level_selected_track(self,send_no,cc_value):
		self.set_send_level_track(send_no,self.selected_track(),cc_value)

	def set_send_level_track(self,send_no,track,cc_value):
		if(send_no<len(track.mixer_device.sends)):
			track.mixer_device.sends[send_no].value=(track.mixer_device.sends[send_no].max - track.mixer_device.sends[send_no].min)	* cc_value / 127 + track.mixer_device.sends[send_no].min

	# MUTE
	def toggle_mute_selected_track(self):
		self.toggle_mute_track(self.selected_track())

	def toggle_mute_track(self,track):
		if(track.mute):
			track.mute=0
		else:
			track.mute=1

	# SOLO
	def toggle_solo_selected_track(self):
		self.toggle_solo_track(self.selected_track())

	def toggle_solo_track(self,track):
		if(track.solo):
			track.solo=0
		else:
			track.solo=1

	# ARMING
	def toggle_arm_selected_track(self):
		self.toggle_arm_track(self.selected_track())

	def unarm_selected_track(self):
		self.unarm_track(self.selected_track())

	def arm_selected_track(self):
		self.arm_track(self.selected_track())

	def toggle_arm_track(self, track):
		if(track.arm):
			self.unarm_track(track)
		else:
			self.arm_track(track)

	def unarm_track(self, track):
		track.arm = 0

	def arm_track(self, track):
		tracks = self.song().tracks #self.find_similar_tracks(track)
		if(EXCLUSIVE_ARM):
			for t in tracks:
				if t.can_be_armed and t.arm:
					t.arm = 0
		track.arm = 1
		
	def select_next_track(self):
		self.select_relative_track(1)
	def select_prev_track(self):
		self.select_relative_track(-1)
	
	def index_of(self,list, elt):
		for i in range(0,len(list)):
			if (list[i] == elt):
				return i
	
	def select_relative_track(self,move):
		if(self.parent.song().view.selected_track!=None):
			idx = self.index_of(self.parent.song().tracks,self.parent.song().view.selected_track)+move
			if( idx<len(self.parent.song().tracks) and idx>=0):
				self.parent.song().view.selected_track=self.parent.song().tracks[idx]

	def disconnect(self):
		pass
	
